
;-------------------------------;
;- Software Sprite Engine v1.5 -;
;-------------------------------;
;- changes: 21.08.2006          ;
;-------------------------------;

/*

 wersja zrownowazona wzgledem szybkosci i pamieciozernosci (obszar pamieci podstawowej $4000..$7FFF jest wolny)
 !!! tylko jedna procedura tworzaca ducha na stronie zerowej >> SHAPEZP << !!!

 sprite: 12x24 pixli na znakach, programy obslugi (B2CALC8, B3CALC8) --> plik SPRITE_8x24.ASM (dodatkowo B2INIT, B3INIT)
 sprite:  8x24 pixli na znakach, programy obslugi (B2CALC, B3CALC) --> plik SPRITE_12x24.ASM

 strona zerowa zajeta jest od adresu ZPAGE przez procedure SHAPEZP i zmienne do ktorych przepisywane sa parametry duchow
 aby dodac nowe zmienne do strony zerowej nalezy zmodyfikowac plik GLOBAL\GLOBAL.HEA (dodac zmienne przez .DS)

 tablice przechowujace parametry duchow i ksztaltow definiuja struktury @SPRITE i @SHAPE (plik DATA.ASM), rozmiar
 tych struktur decyduje o maksymalnej liczbie duchow i maksymalnej liczbie mozliwych ksztaltow, aktualnie
 maksymalna liczba duchow ktora moze przetworzyc silnik to 19, maksymalna liczba ksztaltow to 64 (1 ksztalt = 1 bank)

 dane klatek i masek animacji kazdego ducha niezaleznie od jego rozmiaru umieszczone sa w bankach co $0800 bajtow:

 $4000-$47FF SHP.SHR0		$6000-$67FF MSK.SHR0
 $4800-$4FFF SHP.SHR2		$6800-$6FFF MSK.SHR2
 $5000-$57FF SHP.SHR4		$7000-$77FF MSK.SHR4
 $5800-$5FFF SHP.SHR6		$7800-$7FFF MSK.SHR6

 SHP - ksztalt czyli dane graficzne ducha, te wartosci potrzebne sa dla operacji ORA
 MSK - maska czyli tlo ducha reprezentuje pixle ktore powinny zostac skasowane, te wartosci potrzebne sa dla operacji AND

 SHR0, SHR2, SHR4, SHR6	- przesuniecie grafiki i maski ducha o zadana liczbe bitow (0,2,4,6)

 znaki wykorzystywane przez duchy przydzielane sa dynamicznie, pierwszym wolnym znakiem jest !!! charsBAK !!!
 charsBAK wyznacza liczbe znakow przeznaczonych na tlo <0..charsBAK-1>
 pozostale znaki <charsBAK..127> przeznaczone zostaja na duchy

*/


	icl 'global\global.hea'


@TAB_MEM_BANKS	= $0100		; tablica z kodami bankow dla rejestru $d301 (PORTB)

pmg		= $0000		; adres pamieci dla PMG

prg		= $2000		; adres programu, im nizszy tym wiecej ciaglego obszaru pamieci do wykorzystania

max_shapes	= 256/@SHAPE	; maksymalna dopuszczalna liczba ksztaltow duchow (@SHAPE to struktura danych)
max_sprites	= 256/@SPRITE	; maksymalna dopuszczalna liczba duchow do wyswietlenia (@SPRITE to struktura danych)

min_frames	= 3		; minimalna liczba klatek synchronizacji dla silnika ponizej ktorej nie moze zejsc
				; to na wypadek gdyby silnik przetwarzal mniejsza liczbe duchow, wowczas widoczne bylyby
				; przyspieszenia i zwolnienia albo gdyby silnik byl uruchamiany na szybszej Atarce

ile		= 16-1		; ile duchow wyswietlic, zmienna na potrzeby demonstracji dzialania silnika

	ert ile>max_sprites


* ---	INIT

	org $0600
	.ds 128
@PROC_ADD_BANK			; adres procedury przelaczajacej banki dla LMB, NMB

	org prg

	icl 'global\proc\@mem_detect.asm'

	ini @mem_detect

*---
	org prg
	
init	.link 'init\init.obx'	; inicjalizacja buforow, przepisanie fontow itd.

	ini init


* ---	BANKI PAMIECI DODATKOWEJ
	opt b+				; BANK SENSITIVE ON

; w MADS etykiety zdefiniowane w banku >0 sa zasiegu lokalnego
; nieograniczony dostep do takich etykiet mozliwy jest tylko z poziomu banku 0
;

* ---	PROCEDURY OBSLUGI DUCHA 12x24

*---	obsluga BUFOR #2 (duch 12x24)
	lmb #1				; startujemy od banku #1
	.link 'b2calc_12.obx'
B2CALC	= BCALC			; etykieta lokalna B2CALC dla tego banku

*---	obsluga BUFOR #3 (duch 12x24)
	nmb
	.link 'b3calc_12.obx'
B3CALC	= BCALC			; etykieta lokalna B3CALC dla tego banku


* ---	PROCEDURY OBSLUGI DUCHA 8x24 ORAZ INICJALIZACJI BUFOROW

*---	obsluga BUFOR #2 (duch 8x24)
	nmb
	.pages $40

	.link 'b2calc_8.obx'
B2CALC8	= BCALC			; etykieta lokalna B2CALC8 dla tego banku

B2INIT				; INICJALIZACJA DLA BUFOR #2 (B2SCR)
	lda #$ff
	:@sw*4+12	sta COLISx+#
	:@sh*8+24	sta COLISy+#

	ldx #0
	jsr _init
	inx
_init
	?cnt = 0
	.rept @sw/2
	MOVE	B2
	.endr
	
	rts

	.endpg


*---	obsluga BUFOR #3 (duch 8x24)
	nmb
	.pages $40

	.link 'b3calc_8.obx'
B3CALC8	= BCALC			; etykieta lokalna B3CALC8 dla tego banku

B3INIT				; INICJALIZACJA DLA BUFOR #3 (B3SCR)
	lda #$ff
	:@sw*4+12	sta COLISx+#
	:@sh*8+24	sta COLISy+#

	ldx #0
	jsr _init
	inx
_init
	?cnt = 0
	.rept @sw/2
	MOVE	B3
	.endr
	
	rts

	.endpg


* ---	dane dla SHAPE1 co $800 bajtow
	nmb
	.pages $40
	
.local	shape1

	align $0800
	ins 'shape\shp1s0.dat'
	align $0800
	ins 'shape\shp1s2.dat'
	align $0800
	ins 'shape\shp1s4.dat'
	align $0800
	ins 'shape\shp1s6.dat'

	align $0800
	ins 'shape\msk1s0.dat'
	align $0800
	ins 'shape\msk1s2.dat'
	align $0800
	ins 'shape\msk1s4.dat'
	align $0800
	ins 'shape\msk1s6.dat'

.endl
	.endpg


* ---	dane dla SHAPE2 co $0800 bajtow
	nmb
	.pages $40

.local	shape2

	align $0800
	ins 'shape\shp2s0.dat'
	align $0800
	ins 'shape\shp2s2.dat'
	align $0800
	ins 'shape\shp2s4.dat'
	align $0800
	ins 'shape\shp2s6.dat'

	align $0800
	ins 'shape\msk2s0.dat'
	align $0800
	ins 'shape\msk2s2.dat'
	align $0800
	ins 'shape\msk2s4.dat'
	align $0800
	ins 'shape\msk2s6.dat'

.endl
	.endpg


* ---	dane dla SHAPE3 co $800 bajtow
	nmb
	.pages $40
	
.local	shape3

	align $0800
	ins 'shape\shp3s0.dat'
	align $0800
	ins 'shape\shp3s2.dat'
	align $0800
	ins 'shape\shp3s4.dat'
	align $0800
	ins 'shape\shp3s6.dat'

	align $0800
	ins 'shape\msk3s0.dat'
	align $0800
	ins 'shape\msk3s2.dat'
	align $0800
	ins 'shape\msk3s4.dat'
	align $0800
	ins 'shape\msk3s6.dat'

.endl
	.endpg


	opt b-				; BANK SENSITIVE OFF

	rmb			; BANK = 0

; od teraz mamy dostep do wszystkich etykiet zdefiniowanych w obszarze dodatkowych bankow

* ---	MAIN PROGRAM

	org prg

* ---	ANTIC PROGRAMs

B2ant
	dta d'pp',$f0
	dta $c4,a(B2scr)
	:@sh-2 dta $84
	dta 4
	dta b($41),a(B2ant)

B3ant	dta d'pp',$f0
	dta $c4,a(B3scr)
	:@sh-2 dta $84
	dta 4
	dta b($41),a(B3ant)


* ---	DATA

	icl 'data.asm'


*---
 	align

* ---	DLI PROGRAM
; program przerwania DLI, koniecznie musi miescic sie w granicy strony pamieci
;
; dyrektywa .PAGES wygeneruje ostrzezenie jesli kod zawarty pomiedzy .PAGES i .ENDPG
; znajdzie sie poza obszarem strony pamieci

	.pages

dli0	lda >B2fnt0
	sta $d40a
	sta chbase
	mva <dli1 vdli+1
	lda rA2
	rti

dli1	lda >B2fnt1
	sta $d40a
	sta chbase
	mva <dli2 vdli+1
	lda rA2
	rti

dli2	lda >B2fnt2
	sta $d40a
	sta chbase
	mva <dli3 vdli+1
	lda rA2
	rti

dli3	lda >B2fnt3
	sta $d40a
	sta chbase
	mva <dli0 vdli+1
	lda rA2
	rti

	.endpg


* ---	MAIN PROGRAM
; wylaczamy ROM, inicjalizujemy odpowiednie rejestry
; zaczynamy skokiem do petli JMP LOOP
 
main
	lda:cmp:req 20

	sei
	mva #$00 $d40e
	mva #$fe $d301
 
	mva >pmg $d407		; missiles and players data address
	mva #3 $d01d		; enable players and missiles


; inicjalizacja duchow sprzetowych

	ldy #0			; wyczyszczenie pamieci dla duchow sprzetowych
	tya
clrPMG	sta	pmg+$300,y
	sta	pmg+$400,y
	sta	pmg+$500,y
	sta	pmg+$600,y
	sta	pmg+$700,y
	iny
	bne	clrPMG

	mva #$ff sizem		; pociski najszersze

	lda #1
	sta sizep0
	sta sizep1
	sta sizep2
	sta sizep3

	lda #8
	sta gtictl

	?b = 64
	mva #?b		hposp0
	sta hposm0
	mva #?b+16	hposp1
	sta hposm1
	mva #?b+32	hposp2
	sta hposm2
	mva #?b+48	hposp3
	sta hposm3

	ldx #$1f
kk	lda #$ff
	sta pmg+$300+120,x
	lda #$f8
	sta pmg+$400+60,x
	sta pmg+$500+60,x
	sta pmg+$600+60,x
	sta pmg+$700+60,x
	dex
	bpl kk

	mwa #nmi $fffa
	mwa #B2ant 560
 
 
;---	inicjalizacja silnika

	ldx #$7f		; skasowanie bit0 w kodach bankow pamieci
_cl	lda @TAB_MEM_BANKS,x
	and #$fe
	sta @TAB_MEM_BANKS,x
	dex
	bpl _cl


	jsr shapeMOV		; przepisanie procedury SHAPEZP na strone zerowa


	lda #0			; zerowanie statusu duchow, wszystkie wylaczone STS=0
	:max_sprites sta sprites[#].sts


	SBANK	shape1		; dopisanie ksztaltu SHAPE1 do tablicy SHAPES
	sta	shapes[0].bnk	; bank w ktorym znajduja sie dane ksztaltu
	mva #8	shapes[0].frm	; liczba klatek animacji tego ksztaltu
	mva #0	shapes[0].typ	; =0 ksztalt o rozmiarze 12x24
	mva #21	shapes[0].hig	; wysokosc dla tego ksztaltu

	SBANK	shape2		; dopisanie ksztaltu SHAPE2 do tablicy SHAPES
	sta	shapes[1].bnk	; bank w ktorym znajduja sie dane ksztaltu
	mva #10	shapes[1].frm	; liczba klatek animacji tego ksztaltu
	mva #0	shapes[1].typ	; =0 ksztalt o rozmiarze 12x24
	mva #21	shapes[1].hig	; wysokosc dla tego ksztaltu

	SBANK	shape3		; dopisanie ksztaltu SHAPE1 do tablicy SHAPES
	sta	shapes[2].bnk	; bank w ktorym znajduja sie dane ksztaltu
	mva #13	shapes[2].frm	; liczba klatek animacji tego ksztaltu
	sta	shapes[2].typ	; <>0 ksztalt o rozmiarze 8x24
	mva #14	shapes[2].hig	; wysokosc dla tego ksztaltu


; zaincjowanie tablicy SPRITES, na przemian duch ksztaltu SHAPE1 i duch ksztaltu SHAPE2

	?b = 0
	
	.rept max_sprites

	ift ?b<ile
	mwa #sprite0	sprites[?b].prg	; program obslugi ducha
	mva #0	sprites[?b].new		; wartosc 0 oznacza ducha ktorego powinno obsluzyc sie w tej kolejce i pozostalych
	mva #0  sprites[?b].shp		; duch ksztaltu SHAPE1
	mva #1	sprites[?b].sts		; sts<>0, status mozna wykorzystac do okreslenia rodzaju ducha
	mva #0	sprites[?b].frm
	mva #17+#*12	sprites[?b].psx
	mva #8+#*8	sprites[?b].psy
	eif
	?b++

	ift ?b<ile
	mwa #sprite0	sprites[?b].prg	; program obslugi ducha
	mva #0	sprites[?b].new		; wartosc <>0 oznacza ducha ktorego powinno obslugiwac sie od nastepnej kolejki
	mva #1  sprites[?b].shp		; duch ksztaltu SHAPE2
	mva #1	sprites[?b].sts		; sts<>0, status mozna wykorzystac do okreslenia rodzaju ducha
	mva #0	sprites[?b].frm
	mva #16+#*12	sprites[?b].psx
	mva #32+#*8	sprites[?b].psy
	eif
	?b++

	ift ?b<ile
	mwa #sprite0	sprites[?b].prg	; program obslugi ducha
	mva #0	sprites[?b].new		; wartosc <>0 oznacza ducha ktorego powinno obslugiwac sie od nastepnej kolejki
	mva #2  sprites[?b].shp		; duch ksztaltu SHAPE2
	mva #1	sprites[?b].sts		; sts<>0, status mozna wykorzystac do okreslenia rodzaju ducha
	mva #0	sprites[?b].frm
	mva #<[16+#*12]	sprites[?b].psx
	mva #<[32+#*8]	sprites[?b].psy
	eif
	?b++

	.endr

;---	

	mva #$c0 $d40e
	
;---

	jmp	loop		; LET'S GO


	icl 'loop.asm'

	icl 'detect.asm'


* ---	SPRITE0
; uniwersalna procedura obslugi ducha
;
; do kazdej procedury obslugi przekazywany jest w rejesrze X indeks do tablicy SPRITES
;
; na podstawie rejestru X mozemy odczytac poszczegolne parametry ducha
;
; po tablicy SPRITES poruszamy sie za pomoca indeksow zdefiniowanych w strukturze @SPRITE

sprite0
	lda $d20a
	and #7
	tay
	lda tadd,y
	adc sprites[0].psy,x
	cmp #@sh*8-24
	scc
	lda #0

	sta sprites[0].psy,x

	lda $d20a
	and #7
	tay
	lda tadd,y
	adc sprites[0].psx,x
	cmp #@sw*4-12
	scc
	lda #0

	sta sprites[0].psx,x

	jsr DETECT

	bcc BRAK_KOLIZJI

;	lda $D20a
;	sta $d01a

BRAK_KOLIZJI
	rts

tadd	dta 2,1,1,1,$ff,1,$ff,$fe

* ---	SHAPEZP
; uniwersalna procedura modyfikujaca zestaw znakow (procedure definiuje struktura @ZPVAR)
; SRC -> adresy znakow ktore zostana odczytane i poddane modyfikacji
; MSK -> maska ksztaltu ducha dla operacji AND
; SHP -> maska z ksztaltem ducha dla operacji OR (maska i kszalt nie sa wzgledem siebie "symetryczne")
; DST -> adresy znakow docelowych, tutaj zostanie zapisany wynik operacji LDA:AND:ORA

; shapeTMP zostanie przepisany na strone zerowa od adresu shapeZP

shapeMOV
	ldx	#0
mov	mva	shapeTMP,x	shapeZP,x+
	cpx	#shapeTMP_end-shapeTMP
	bne	mov
	rts

*---

shapeTMP	dta @ZPVAR [15] ( {lda*,y}, $ffff, {and*,y}, $ffff, {ora*,y}, $ffff, {sta*,y}, $ffff )

	dey
	smi
	jmp shapeZP
	rts

shapeTMP_end

*---

* ---	NMI
	
nmi	bit $d40f
	bpl vbl

	sta rA2
vdli	jmp dli0

vbl	sta rA
	sta $d40f

	inc cloc

	mwa 560 $d402

	ift @sw=32
	 lda #%00111101
	eli @sw=40
	 lda #%00111110
	els
	 lda #%00111111
	eif

	sta $d400

	mva	#$14	color0
	mva	#8	color1
	mva	#$f	color2
	mva	#$fe	color3
	mva	#0	colbak

	mva	#$18	colpm0
	mva	#$28	colpm1
	mva	#$38	colpm2
	mva	#$48	colpm3

	mva <dli0 vdli+1

	lda rA
	rti


* ---	VARIABLE

cloc	brk


* ---	RUN

	.print 'FREE MEM: ',*,'..$BFFF, ',free_mem,'..$FFF0'
	.print 'FREE ZPAGE: ',freeZP,'..$00FF'

	run main


	opt l-
	icl 'global\align.mac'
	icl 'global\@bank_add.mac'


SBANK	.macro
	ldx #=:1
	lda @TAB_MEM_BANKS,x
	.endm


MOVE	.macro 
	:@sh	mva B1scr+#*@sw+?cnt,x	:1scr+#*@sw+?cnt,x
	?cnt+=2
	.endm
